@@ -241,6 +241,8 @@ hash_public_key( MD_HANDLE md, PKT_public_key *pk )
 {
     PACKET pkt;
     int rc = 0;
+    int ctb;
+    ulong pktlen;
     int c;
     IOBUF a = iobuf_temp();
   #if 0
@@ -256,6 +258,42 @@ hash_public_key( MD_HANDLE md, PKT_public_key *pk )
     pkt.pkt.public_key = pk;
     if( (rc = build_packet( a, &pkt )) )
 	log_fatal("build public_key for hashing failed: %s\n", g10_errstr(rc));
+
+    if( !(pk->version == 3 && pk->pubkey_algo == 16) ) {
+	/* skip the constructed header but don't do this for our very old
+	 * v3 ElG keys */
+	ctb = iobuf_get_noeof(a);
+	pktlen = 0;
+	if( (ctb & 0x40) ) {
+	    c = iobuf_get_noeof(a);
+	    if( c < 192 )
+		pktlen = c;
+	    else if( c < 224 ) {
+		pktlen = (c - 192) * 256;
+		c = iobuf_get_noeof(a);
+		pktlen += c + 192;
+	    }
+	    else if( c == 255 ) {
+		pktlen	= iobuf_get_noeof(a) << 24;
+		pktlen |= iobuf_get_noeof(a) << 16;
+		pktlen |= iobuf_get_noeof(a) << 8;
+		pktlen |= iobuf_get_noeof(a);
+	    }
+	}
+	else {
+	    int lenbytes = ((ctb&3)==3)? 0 : (1<<(ctb & 3));
+	    for( ; lenbytes; lenbytes-- ) {
+		pktlen <<= 8;
+		pktlen |= iobuf_get_noeof(a);
+	    }
+	}
+	/* hash a header */
+	md_putc( md, 0x99 );
+	pktlen &= 0xffff; /* can't handle longer packets */
+	md_putc( md, pktlen >> 8 );
+	md_putc( md, pktlen & 0xff );
+    }
+    /* hash the packet body */
     while( (c=iobuf_get(a)) != -1 ) {
       #if 0
 	fprintf( fp," %02x", c );
@@ -309,7 +347,7 @@ do_secret_key( IOBUF out, int ctb, PKT_secret_key *sk )
 	if( is_RSA(sk->pubkey_algo) && sk->version < 4
 				    && !sk->protect.s2k.mode ) {
 	    iobuf_put(a, sk->protect.algo );
-	    iobuf_write(a, sk->protect.iv, 8 );
+	    iobuf_write(a, sk->protect.iv, sk->protect.ivlen );
 	}
 	else {
 	    iobuf_put(a, 0xff );
@@ -321,7 +359,7 @@ do_secret_key( IOBUF out, int ctb, PKT_secret_key *sk )
 		iobuf_write(a, sk->protect.s2k.salt, 8 );
 	    if( sk->protect.s2k.mode == 3 )
 		iobuf_put(a, sk->protect.s2k.count );
-	    iobuf_write(a, sk->protect.iv, 8 );
+	    iobuf_write(a, sk->protect.iv, sk->protect.ivlen );
 	}
     }
     else
@@ -694,9 +732,15 @@ do_signature( IOBUF out, int ctb, PKT_signature *sig )
     }
     iobuf_put(a, sig->digest_start[0] );
     iobuf_put(a, sig->digest_start[1] );
-    n = pubkey_get_nsig( sig->pubkey_algo );
-    if( !n )
-	write_fake_data( a, sig->data[0] );
+    n = sig->pubkey_algo? pubkey_get_nsig( sig->pubkey_algo ) : 0;
+    if( !n ) {	/* the MDC data */
+	fputs("The MDC: ", stderr);
+	mpi_print(stderr, sig->data[0], 0 );
+	fputs("  ", stderr);
+	mpi_print(stderr, sig->data[0], 1 );
+	putc('\n', stderr);
+	mpi_write( a, sig->data[0] );
+    }
     for(i=0; i < n; i++ )
 	mpi_write(a, sig->data[i] );
 
