@@ -1,5 +1,5 @@
 /* parse-packet.c  - read packets
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -1024,7 +1024,7 @@ parse_signature( IOBUF inp, int pkttype, unsigned long pktlen,
     sig->digest_start[0] = iobuf_get_noeof(inp); pktlen--;
     sig->digest_start[1] = iobuf_get_noeof(inp); pktlen--;
 
-    if( is_v4 ) { /*extract required information */
+    if( is_v4 && sig->pubkey_algo ) { /*extract required information */
 	const byte *p;
 
 	/* set sig->flags.unknown_critical if there is a
@@ -1065,24 +1065,36 @@ parse_signature( IOBUF inp, int pkttype, unsigned long pktlen,
 	}
     }
 
-    ndata = pubkey_get_nsig(sig->pubkey_algo);
-    if( !ndata ) {
-	if( list_mode )
-	    printf("\tunknown algorithm %d\n", sig->pubkey_algo );
-	unknown_pubkey_warning( sig->pubkey_algo );
-	/* we store the plain material in data[0], so that we are able
-	 * to write it back with build_packet() */
-	sig->data[0] = read_rest(inp, &pktlen );
+    if( !sig->pubkey_algo ) {
+	n = pktlen;
+	sig->data[0] = mpi_read(inp, &n, 0 );
+	pktlen -=n;
+	if( list_mode ) {
+	    printf("\tMDC data: ");
+	    mpi_print(stdout, sig->data[0], mpi_print_mode );
+	    putchar('\n');
+	}
     }
     else {
-	for( i=0; i < ndata; i++ ) {
-	    n = pktlen;
-	    sig->data[i] = mpi_read(inp, &n, 0 );
-	    pktlen -=n;
-	    if( list_mode ) {
-		printf("\tdata: ");
-		mpi_print(stdout, sig->data[i], mpi_print_mode );
-		putchar('\n');
+	ndata = pubkey_get_nsig(sig->pubkey_algo);
+	if( !ndata ) {
+	    if( list_mode )
+		printf("\tunknown algorithm %d\n", sig->pubkey_algo );
+	    unknown_pubkey_warning( sig->pubkey_algo );
+	    /* we store the plain material in data[0], so that we are able
+	     * to write it back with build_packet() */
+	    sig->data[0] = read_rest(inp, &pktlen );
+	}
+	else {
+	    for( i=0; i < ndata; i++ ) {
+		n = pktlen;
+		sig->data[i] = mpi_read(inp, &n, 0 );
+		pktlen -=n;
+		if( list_mode ) {
+		    printf("\tdata: ");
+		    mpi_print(stdout, sig->data[i], mpi_print_mode );
+		    putchar('\n');
+		}
 	    }
 	}
     }
@@ -1305,29 +1317,38 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 	    }
 	    else { /* old version; no S2K, so we set mode to 0, hash MD5 */
 		sk->protect.s2k.mode = 0;
-		/* We need a kludge to cope with old GNUPG versions */
-		sk->protect.s2k.hash_algo =
-			     ( sk->protect.algo == CIPHER_ALGO_BLOWFISH160
-			       && algorithm == PUBKEY_ALGO_ELGAMAL_E ) ?
-				      DIGEST_ALGO_RMD160 : DIGEST_ALGO_MD5;
+		sk->protect.s2k.hash_algo = DIGEST_ALGO_MD5;
 		if( list_mode )
 		    printf(  "\tprotect algo: %d  (hash algo: %d)\n",
 			 sk->protect.algo, sk->protect.s2k.hash_algo );
 	    }
-	    if( pktlen < 8 ) {
+	    /* It is really ugly that we don't know the size
+	     * of the IV here in cases we are not aware of the algorithm.
+	     * so a
+	     *	 sk->protect.ivlen = cipher_get_blocksize(sk->protect.algo);
+	     * won't work.  The only solution I see is to hardwire it here.
+	     */
+	    switch( sk->protect.algo ) {
+	      case 7: case 8: case 9: /* reserved for AES */
+	      case 10: /* Twofish */
+		sk->protect.ivlen = 16;
+		break;
+	      default:
+		sk->protect.ivlen = 8;
+	    }
+	    if( pktlen < sk->protect.ivlen ) {
 		rc = G10ERR_INVALID_PACKET;
 		goto leave;
 	    }
-	    /* fixme: Add support for other blocksizes */
-	    for(i=0; i < 8 && pktlen; i++, pktlen-- )
+	    for(i=0; i < sk->protect.ivlen && pktlen; i++, pktlen-- )
 		temp[i] = iobuf_get_noeof(inp);
 	    if( list_mode ) {
 		printf(  "\tprotect IV: ");
-		for(i=0; i < 8; i++ )
+		for(i=0; i < sk->protect.ivlen; i++ )
 		    printf(" %02x", temp[i] );
 		putchar('\n');
 	    }
-	    memcpy(sk->protect.iv, temp, 8 );
+	    memcpy(sk->protect.iv, temp, sk->protect.ivlen );
 	}
 	else
 	    sk->is_protected = 0;
@@ -1335,7 +1356,7 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 	 * If the user is so careless, not to protect his secret key,
 	 * we can assume, that he operates an open system :=(.
 	 * So we put the key into secure memory when we unprotect it. */
-	if( is_v4 && sk->is_protected ){
+	if( is_v4 && sk->is_protected ) {
 	    /* ugly; the length is encrypted too, so we read all
 	     * stuff up to the end of the packet into the first
 	     * skey element */
@@ -1544,14 +1565,14 @@ parse_encrypted( IOBUF inp, int pkttype, unsigned long pktlen,
     ed->len = pktlen;
     ed->buf = NULL;
     ed->new_ctb = new_ctb;
-    if( pktlen && pktlen < 10 ) {
+    if( pktlen && pktlen < 10 ) { /* actually this is blocksize+2 */
 	log_error("packet(%d) too short\n", pkttype);
 	skip_rest(inp, pktlen);
 	goto leave;
     }
     if( list_mode ) {
 	if( pktlen )
-	    printf(":encrypted data packet:\n\tlength: %lu\n", pktlen-10);
+	    printf(":encrypted data packet:\n\tlength: %lu\n", pktlen);
 	else
 	    printf(":encrypted data packet:\n\tlength: unknown\n");
     }
