@@ -411,8 +411,11 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 		md_write( md, sig->hashed_data, n+2 );
 		n += 6;
 	    }
-	    else
+	    else {
+		md_putc( md, 0 );  /* always hash the length of the subpacket*/
+		md_putc( md, 0 );
 		n = 6;
+	    }
 	    /* add some magic */
 	    buf[0] = sig->version;
 	    buf[1] = 0xff;
@@ -594,8 +597,11 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 		md_write( md, sig->hashed_data, n+2 );
 		n += 6;
 	    }
-	    else
+	    else {
+		md_putc( md, 0 );  /* always hash the length of the subpacket*/
+		md_putc( md, 0 );
 		n = 6;
+	    }
 	    /* add some magic */
 	    buf[0] = sig->version;
 	    buf[1] = 0xff;
@@ -656,7 +662,8 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
     MD_HANDLE md;
 
     assert( (sigclass >= 0x10 && sigclass <= 0x13)
-	    || sigclass == 0x20 || sigclass == 0x18 );
+	    || sigclass == 0x20 || sigclass == 0x18
+	    || sigclass == 0x30 || sigclass == 0x28 );
     if( !digest_algo ) {
 	switch( sk->pubkey_algo ) {
 	  case PUBKEY_ALGO_DSA: digest_algo = DIGEST_ALGO_SHA1; break;
@@ -669,7 +676,7 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
 
     /* hash the public key certificate and the user id */
     hash_public_key( md, pk );
-    if( sigclass == 0x18 ) { /* subkey binding */
+    if( sigclass == 0x18 || sigclass == 0x28 ) { /* subkey binding/revocation*/
 	hash_public_key( md, subpk );
     }
     else if( sigclass != 0x20 ) {
@@ -720,8 +727,11 @@ make_keysig_packet( PKT_signature **ret_sig, PKT_public_key *pk,
 		md_write( md, sig->hashed_data, n+2 );
 		n += 6;
 	    }
-	    else
+	    else {
+		md_putc( md, 0 );  /* always hash the length of the subpacket*/
+		md_putc( md, 0 );
 		n = 6;
+	    }
 	    /* add some magic */
 	    buf[0] = sig->version;
 	    buf[1] = 0xff;
