@@ -1,5 +1,5 @@
 /* ttyio.h
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GNUPG.
  *
@@ -20,11 +20,13 @@
 #ifndef G10_TTYIO_H
 #define G10_TTYIO_H
 
+int tty_batchmode( int onoff );
 void tty_printf( const char *fmt, ... );
 void tty_print_string( byte *p, size_t n );
 char *tty_get( const char *prompt );
 char *tty_get_hidden( const char *prompt );
 void tty_kill_prompt(void);
+int tty_get_answer_is_yes( const char *prompt );
 
 
 #endif /*G10_TTYIO_H*/
